module TwoFactorDevices
  # Base class for 2fa devices
  class TwoFactorDevice < ApplicationRecord
    VALUE_MASK = /\A\z/.freeze
    self.table_name = 'two_factor_devices'.freeze

    belongs_to :owner, class_name: TwoFactorDevices.device_owner_class_name

    scope :confirmed, -> { where(is_confirmed: true) }
    scope :default, -> { where(is_primary: true) }
    scope :backup, -> { where(is_primary: false) }

    validate :disallow_multiple_primary_devices, if: :is_primary?

    def mark_as_primary!
      with_lock do
        owner.two_factor_devices.default.update_all(is_primary: false)

        self.is_primary = true
        save!
      end
    end

    def confirm!
      self.is_confirmed = true
      save!
    end

    def masked_value
      value.gsub(value_mask, '*')
    end

    private

    def value_mask
      self.class::VALUE_MASK
    end

    def disallow_multiple_primary_devices
      return unless self.class.default.where(owner_id: owner_id).where.not(id: id).exists?

      errors.add(:is_primary,
                 I18n.t('activerecord.errors.models.two_factor_device.attributes.is_primary.more_than_one_primary'))
    end
  end
end
