module TwoFactorDevices
  # Generator for simple email gateway integration to project
  # 1. Adds mailer class with associated template
  # 2. Create gateway class, that uses mailer for otp code transmission
  class EmailGatewayGenerator < Rails::Generators::NamedBase
    check_class_collision suffix: 'Mailer'
    source_root File.expand_path('templates', __dir__)

    desc 'Adds otp mailer to project.'
    def create_mailer
      template 'otp_mailer.rb', File.join('app', 'mailers', class_path, "#{file_name}_mailer.rb")
    end

    def copy_templates
      copy_file 'otp_email.html.erb',
                File.join('app', 'views', 'two_factor_device_authentication', 'otp_email.html.erb')
    end

    def create_gateway
      template 'email_gateway.rb', File.join('lib', 'two_factor_devices', 'gateways', 'email.rb')
    end

    private

    def file_name
      super.sub(/_mailer\z/i, '')
    end
  end
end
