require 'rails/generators/active_record'

module TwoFactorDevices
  # Installation generator
  # 1. Copy migration to project
  # 2. Include gem extension to device's owner model
  # 3. Setup TwoFactotdevices initializer
  # 4. Allow to setup email gateway if needed
  class InstallGenerator < Rails::Generators::NamedBase
    include ActiveRecord::Generators::Migration
    source_root File.expand_path('templates', __dir__)

    desc 'Adds migration'
    def create_migrations
      migration_template File.join('migrations', 'device_table.rb.tt'),
                         File.join('db', 'migrate', 'create_two_factor_devices.rb'),
                         migration_version: migration_version
    end

    def include_device_relation
      path = File.join('app', 'models', "#{file_path}.rb")
      return unless File.exist?(path)

      inject_into_class(path, class_name, "  include TwoFactorDevices::Extensions::TwoFactorDeviceble\n")
    end

    def copy_initializer
      template 'initializer.rb', File.join('config', 'initializers', 'two_factor_devices.rb')
    end

    def ask_about_mailer
      return unless yes?('Do you want to setup email gateway via rails mailer? [yes/no]')

      mailer_name = ask('How you prefer to name mailer? [OtpMailer]').presence || 'OtpMailer'
      generate 'two_factor_devices:email_gateway', mailer_name
      uncomment_lines(
        File.join('config', 'initializers', 'two_factor_devices.rb'),
        /config.email_gateway/
      )
    end

    private

    def migration_version
      "[#{ActiveRecord::VERSION::MAJOR}.#{ActiveRecord::VERSION::MINOR}]"
    end
  end
end
