require 'two_factor_devices/version'
require 'two_factor_devices/configuration'
require 'two_factor_devices/engine'
require 'two_factor_devices/extensions/two_factor_deviceble'
require 'two_factor_devices/gateways/fake'
require 'two_factor_devices/devices/manager'
require 'two_factor_devices/devices/registrator'
require 'two_factor_devices/devices/validator'
require 'two_factor_devices/gateway'

# mail module
module TwoFactorDevices
  class Error < StandardError; end

  class InvalidDevice < Error; end

  class SendOtpError < Error; end

  class EmailOtpError < SendOtpError; end

  class PhoneOtpError < SendOtpError; end

  extend SingleForwardable

  def_delegators :configuration, :email_gateway, :phone_gateway, :device_owner, :otp_base_secret, :layout

  EMAIL_TYPE = 'email'.freeze
  PHONE_TYPE = 'phone'.freeze
  DEVICE_TYPES = {
    EMAIL_TYPE => 'TwoFactorDevices::EmailDevice'.freeze,
    PHONE_TYPE => 'TwoFactorDevices::PhoneDevice'.freeze
  }.freeze

  # Pre configuration for gateway definition
  def self.configure
    yield configuration
  end

  # Returns an existing or instantiates a new configuration object.
  def self.configuration
    @configuration ||= Configuration.new
  end

  def self.available_types
    DEVICE_TYPES.keys.select { |k| gateway(k).present? }
  end

  def self.gateway(device_type)
    raise ArgumentError, "Unknown device type: #{device_type}." unless DEVICE_TYPES.key?(device_type)

    public_send("#{device_type}_gateway")
  end

  def self.device_manager
    @device_manager ||= Devices::Manager.new
  end

  def self.device_owner_class_name
    raise Error, 'TwoFactorDevice owner not configured. Please set TwoFactorDevices.device_owner.' if device_owner.nil?

    device_owner.to_s
  end

  private_class_method :configuration
end
