require 'rotp'

module TwoFactorDevices
  module Devices
    # logic for device confirmation
    # Allow to generate otp code challenge and validate received code
    class Validator
      DEFAULT_CONFIRMATION_DRIFT = 60

      def make_challenge(device)
        Gateway.send_otp(device, totp.now)
      end

      def validate(code)
        totp.verify(code, drift_behind: DEFAULT_CONFIRMATION_DRIFT)
      end

      def totp
        @totp ||= ROTP::TOTP.new(TwoFactorDevices.otp_base_secret)
      end
    end
  end
end
